* SCRXLIM and SCRYLIM functions, from the
* Display Extensions V2.02, last update 11/11/03
* QLiberator directive $$asmb=flp1_display_cde,0,10
*
* Brief description and syntax of basic extensions:
*
* The FLIM_n functions use iop.flim trap to find window limits. For
* primary channels, this is screen size, for secondaries this is the
* outline limits
*
* SCRXLIM and SCRYLIM return limits of width and height respectively
* LET wide = SCRXLIM(#channel)
* LET high = SCRYLIM(#channel)

* link into basic as extensions
        lea     exts,a1         ;point to list of extensions
        move.w  $110,a2         ;bp.init
        jmp     (a2)            ;link extensions

* list of extension names and definitions
exts    dc.w    0               ;no procedures
        dc.w    0               ;end of procedures list
        
        dc.w    2               ;2 functions
        dc.w    scrxlim-*       ;location of SCRXLIM
        dc.b    7,'SCRXLIM'     ;name of function
        dc.w    scrylim-*       ;location of SCRYLIM
        dc.b    7,'SCRYLIM'     ;name of function
        dc.w    0               ;end of functions
* LET w=SCRXLIM(#channel)
* LET h=SCRYLIM(#channel)

* routines need a 4 byte storage area required by iop.flim for
* width,height,x,y
store   ds.w    4

scrxlim move.w  #512,d5         ;default width
        moveq.l #0,d6           ;offset of width value from "store"
        bra.s   flim            ;branch to common part of routine

scrylim move.w  #256,d5         ;default height
        moveq.l #2,d6           ;offset of height value from "store"

* common routine, with pre-assigned values in d5+d6
flim    move.l  d6,-(a7)        ;ca.gtlin smashes d6 offset
        move.w  $118,a2         ;ca.gtlin
        jsr     (a2)            ;fetch 1 parameter
        move.l  (a7)+,d6        ;recover d6 offset value
        tst.w   d0              ;any error?
        bne.s   return          ;oops!
        subq.w  #1,d3           ;1 parameter
        bne.s   badparam        ;oops!
* we now have the channel number on the stack
        moveq.l #2,d7           ;2 more bytes will be needed
        move.l  0(a6,a1.l),d0   ;get channel number to d0
        bmi     retfp1          ;if negative, return as function value
        bsr     chan2id         ;channel # to ID in a0
        bmi     retfp1          ;oops,return the -ve number as fn value

        moveq   #-1,d3          ;timeout
        clr.l   d2              ;has to be zero
        move.l  a1,$58(a6)      ;store RI stack pointer
        lea.l   store,a1        ;point to 8 byte area
        moveq   #$6C,d0         ;iop.flim
        trap    #3
        move.l  $58(a6),a1      ;recover RI stack pointer
        tst.w   d0              ;was there any error?
        beq.s   noerr           ;no, fetch value from store to return
* since iop.flim appears not to exist, we'll assume a 512x256 display
        move.w  d5,d0           ;the default value for this function
        bra.s   extnd           ;and return that default value
* fetch values for return
noerr   lea.l   store,a2        ;point to four word store area
        move.w  0(a2,d6.l),d0   ;fetch value
extnd   ext.l   d0              ;convert to long word
        bra     retfp1          ;return as floating point value
* utility routines shared by functions
badparam moveq  #-15,d0         ;err.bp
return  rts                     ;go back to basic with d0 error report
notopen moveq   #-6,d0          ;err.no
        rts

* convert and return long word in d0 into a floating point number and
* return (used with permission of Simon N. Goodwin).
* d7.w contains no. of extra bytes required on RI stack over and above
* what's on the RI stack parameters
retfp1  move.l  a1,$58(a6)      ;store RI stack pointer if relevant
retfp   move.w  d0,d1           ;d1 will be exponent
        move.l  d0,d2           ;d2 will be mantissa
        beq.s   normalised      ;zero is a trivial case
        move.w  #2079,d1        ;first guess at exponent
        add.l   d0,d0           ;already normalised?
        bvs.s   normalised
        subq.w  #1,d1           ;no, halve exponent weight
        move.l  d0,d2           ;double mantissa to match
        moveq   #16,d3          ;try a 16 bit shift
normalise move.l d2,d0          ;take copy of mantissa
        asl.l   d3,d0           ;shift mantissa d3 places
        bvs.s   too_far         ;overflow - must shift less
        sub.w   d3,d1           ;correct exponent for shift
        move.l  d0,d2           ;new mantissa is more normal
too_far asr.w   #1,d3           ;halve shift distance
        bne.s   normalise       ;try shift of 8,4,2 and 1
* is extra space required on RI stack? d7 holds number of bytes
* needed over and above existing parameter space
* need to store anything important not preserved by bv.chrix call
normalised movem.l d1-d2,-(a7)  ;store exponent/mantissa
        tst.l   d7              ;if non-zero, create more space
        beq.s   enough          ;no more needed
        move.l  d7,d1           ;no. of extra bytes needed
        move.w  $11A,a0         ;bv.chrix vector
        jsr     (a0)            ;do it
        move.l  $58(a6),a1      ;recover ri pointer
        sub.w   d7,a1           ;adjust by no. of bytes
        move.l  a1,$58(a6)      ;additional bytes pointed to now
enough  movem.l (a7)+,d1-d2     ;recover exponent/mantissa
        move.l  d2,2(a1,a6.l)   ;stack mantissa
        move.w  d1,0(a1,a6.l)   ;stack exponent
        moveq   #2,d4           ;signal FP result
        moveq   #0,d0           ;no errors PLEASE!
        rts

* subroutine to convert channel # in d0 to 4-byte channel ID in a0
chan2id tst.l   d0              ;can't be negative
        bmi.s   retfp1          ;oops, was negative...
        mulu    #$28,d0         ;convert into channel table offset
        add.l   $30(a6),d0      ;add start of table
        cmp.l   $34(a6),d0      ;past end of table?
        bge     notopen         ;yes, channel not open
        movea.l 0(a6,d0.l),a0   ;no-get ID into a0.l
        tst.l   0(a6,d0.l)      ;is channel open?
        bmi     notopen         ;no, return err.no
        moveq   #0,d0           ;yes, return OK code
        rts                     ;and exit
